@@ -1,3 +1,8 @@
+'''Contacting the work unit server.
+
+A Tracker refers to the Universal Tracker
+(https://github.com/ArchiveTeam/universal-tracker).
+'''
 import json
 import functools
 import datetime
@@ -14,6 +19,7 @@ from seesaw.externalprocess import RsyncUpload, CurlUpload
 
 
 class TrackerRequest(Task):
+    '''Represents a request to a Tracker.'''
     def __init__(self, name, tracker_url, tracker_command, may_be_canceled=False):
         Task.__init__(self, name)
         self.http_client = AsyncHTTPClient()
@@ -69,6 +75,7 @@ class TrackerRequest(Task):
 
 
 class GetItemFromTracker(TrackerRequest):
+    '''Get a single work unit information from the Tracker.'''
     def __init__(self, tracker_url, downloader, version=None):
         TrackerRequest.__init__(self, "GetItemFromTracker", tracker_url, "request", may_be_canceled=True)
         self.downloader = downloader
@@ -93,6 +100,7 @@ class GetItemFromTracker(TrackerRequest):
 
 
 class SendDoneToTracker(TrackerRequest):
+    '''Inform the Tracker the work unit has been completed.'''
     def __init__(self, tracker_url, stats):
         TrackerRequest.__init__(self, "SendDoneToTracker", tracker_url, "done")
         self.stats = stats
@@ -110,6 +118,7 @@ class SendDoneToTracker(TrackerRequest):
 
 
 class PrepareStatsForTracker(SimpleTask):
+    '''Apply statistical values on the item.'''
     def __init__(self, defaults=None, file_groups=None, id_function=None):
         SimpleTask.__init__(self, "PrepareStatsForTracker")
         self.defaults = defaults or {}
@@ -133,6 +142,14 @@ class PrepareStatsForTracker(SimpleTask):
 
 
 class UploadWithTracker(TrackerRequest):
+    '''Upload work unit results.
+
+    One of the inner task is used depending on the Tracker's response
+    to where to upload:
+
+    * :class:`RsyncUpload`
+    * :class:`CurlUpload`
+    '''
     def __init__(self, tracker_url, downloader, files, version=None, rsync_target_source_path="./", rsync_bwlimit="0", rsync_extra_args=[], curl_connect_timeout="60", curl_speed_limit="1", curl_speed_time="900"):
         TrackerRequest.__init__(self, "Upload", tracker_url, "upload")
 
@@ -159,23 +176,34 @@ class UploadWithTracker(TrackerRequest):
         if "upload_target" in data:
             inner_task = None
 
-            if re.match(r"^rsync://", data["upload_target"]):
+            if re.match(r"^rsync://.+/$", data["upload_target"]):
                 item.log_output("Uploading with Rsync to %s" % data["upload_target"])
-                inner_task = RsyncUpload(data["upload_target"], self.files, target_source_path=self.rsync_target_source_path, bwlimit=self.rsync_bwlimit, extra_args=self.rsync_extra_args, max_tries=1)
+                inner_task = RsyncUpload(
+                    data["upload_target"], self.files,
+                    target_source_path=self.rsync_target_source_path,
+                    bwlimit=self.rsync_bwlimit,
+                    extra_args=self.rsync_extra_args,
+                    max_tries=1)
 
-            elif re.match(r"^https?://", data["upload_target"]):
+            elif re.match(r"^https?://.+/$", data["upload_target"]):
                 item.log_output("Uploading with Curl to %s" % data["upload_target"])
 
                 if len(self.files) != 1:
                     item.log_output("Curl expects to upload a single file.")
-                    self.fail_item(item)
+                    item.log_output("Contact a tracker admin!")
+                    self.schedule_retry(item)
                     return
 
-                inner_task = CurlUpload(data["upload_target"], self.files[0], self.curl_connect_timeout, self.curl_speed_limit, self.curl_speed_time, max_tries=1)
+                inner_task = CurlUpload(
+                    data["upload_target"], self.files[0],
+                    self.curl_connect_timeout, self.curl_speed_limit,
+                    self.curl_speed_time, max_tries=1)
 
             else:
-                item.log_output("Received invalid upload type.")
-                self.fail_item(item)
+                item.log_output("Received invalid upload URI {0}."
+                                .format(data["upload_target"]))
+                item.log_output("Contact a tracker admin!")
+                self.schedule_retry(item)
                 return
 
             inner_task.on_complete_item += self._inner_task_complete_item
